@@ -97,6 +97,7 @@ class SOCol(object):
                  sqlType=None,
                  columnDef=None,
                  validator=None,
+                 validator2=None,
                  immutable=False,
                  cascade=None,
                  lazy=False,
@@ -193,11 +194,15 @@ class SOCol(object):
             self.alternateMethodName = alternateMethodName
 
         _validators = self.createValidators()
-        if _validators:
-            if validator: _validators.append(validator)
+        if validator: _validators.append(validator)
+        if validator2: _validators.insert(0, validator2)
+        _vlen = len(_validators)
+        if _vlen == 0:
+            self.validator = None # Set sef.{from,to}_python
+        elif _vlen == 1:
+            self.validator = _validators[0]
+        elif _vlen > 1:
             self.validator = compound.All.join(_validators[0], *_validators[1:])
-        else:
-            self.validator = validator
         self.noCache = noCache
         self.lazy = lazy
         # this is in case of ForeignKey, where we rename the column
@@ -453,7 +458,7 @@ class SOStringLikeCol(SOCol):
 
     def _sqliteType(self):
         self._check_case_sensitive("SQLite")
-        return super(SOStringLikeCol, self)._sqliteType().replace("CHAR(", "CHAR (")
+        return super(SOStringLikeCol, self)._sqliteType()
 
     def _sybaseType(self):
         self._check_case_sensitive("SYBASE")
@@ -530,15 +535,21 @@ class StringCol(Col):
 
 class UnicodeStringValidator(validators.Validator):
 
+    def getDbEncoding(self, state):
+        try:
+            return self.dbEncoding
+        except AttributeError:
+            return self.soCol.getDbEncoding(state)
+
     def to_python(self, value, state):
         if value is None:
             return None
         if isinstance(value, (unicode, sqlbuilder.SQLExpression)):
             return value
         if isinstance(value, str):
-            return unicode(value, self.dbEncoding)
+            return unicode(value, self.getDbEncoding(state))
         if isinstance(value, array): # MySQL
-            return unicode(value.tostring(), self.dbEncoding)
+            return unicode(value.tostring(), self.getDbEncoding(state))
         if hasattr(value, '__unicode__'):
             return unicode(value)
         raise validators.Invalid("expected a str or a unicode in the UnicodeCol '%s', got %s %r instead" % \
@@ -550,22 +561,37 @@ class UnicodeStringValidator(validators.Validator):
         if isinstance(value, (str, sqlbuilder.SQLExpression)):
             return value
         if isinstance(value, unicode):
-            return value.encode(self.dbEncoding)
+            return value.encode(self.getDbEncoding(state))
         if hasattr(value, '__unicode__'):
-            return unicode(value).encode(self.dbEncoding)
+            return unicode(value).encode(self.getDbEncoding(state))
         raise validators.Invalid("expected a str or a unicode in the UnicodeCol '%s', got %s %r instead" % \
             (self.name, type(value), value), value, state)
 
 class SOUnicodeCol(SOStringLikeCol):
     def __init__(self, **kw):
-        self.dbEncoding = kw.pop('dbEncoding', 'UTF-8')
+        self.dbEncoding = kw.pop('dbEncoding', None)
         super(SOUnicodeCol, self).__init__(**kw)
 
     def createValidators(self):
-        return [UnicodeStringValidator(name=self.name,
-                dbEncoding=self.dbEncoding)] + \
+        return [UnicodeStringValidator(name=self.name, soCol=self)] + \
             super(SOUnicodeCol, self).createValidators()
 
+    def getDbEncoding(self, state):
+        if self.dbEncoding:
+            return self.dbEncoding
+        dbEncoding = state.soObject.sqlmeta.dbEncoding
+        if dbEncoding:
+            return dbEncoding
+        try:
+            connection = state.connection or state.soObject._connection
+        except AttributeError:
+            dbEncoding = None
+        else:
+            dbEncoding = getattr(connection, "dbEncoding", None)
+        if not dbEncoding:
+            dbEncoding = "utf-8"
+        return dbEncoding
+
 class UnicodeCol(Col):
     baseClass = SOUnicodeCol
 
@@ -948,8 +974,7 @@ class SOEnumCol(SOCol):
         checkConstraint = "CHECK (%s in (%s))" % (self.dbName, enumValues)
         return "VARCHAR(%i) %s" % (length, checkConstraint)
 
-    def _sqliteType(self):
-        return self._postgresType().replace("CHAR(", "CHAR (")
+    _sqliteType = _postgresType
 
     def _sybaseType(self):
         return self._postgresType()
@@ -1131,6 +1156,7 @@ class SODateTimeCol(SOCol):
 
 class DateTimeCol(Col):
     baseClass = SODateTimeCol
+    @staticmethod
     def now():
         if default_datetime_implementation == DATETIME_IMPLEMENTATION:
             return datetime.datetime.now()
@@ -1140,7 +1166,6 @@ class DateTimeCol(Col):
             assert 0, ("No datetime implementation available "
                        "(DATETIME_IMPLEMENTATION=%r)"
                        % DATETIME_IMPLEMENTATION)
-    now = staticmethod(now)
 
 
 class DateValidator(DateTimeValidator):
@@ -1555,7 +1580,7 @@ class PickleCol(BLOBCol):
 
 
 def pushKey(kw, name, value):
-    if not kw.has_key(name):
+    if not name in kw:
         kw[name] = value
 
 all = []
