@@ -1,5 +1,5 @@
 """
-SQLObject 1.0.3
+SQLObject 1.1.2
 ---------------
 
 :author: Ian Bicking <ianb@colorstudy.com>
@@ -40,7 +40,6 @@ import classregistry
 import declarative
 import events
 from sresults import SelectResults
-from formencode import schema, compound
 from util.threadinglocal import local
 
 import sys
@@ -93,9 +92,9 @@ def makeProperties(obj):
         elif var.startswith('_doc_'):
             props.setdefault(var[5:], {})['doc'] = value
     for var, setters in props.items():
-        if len(setters) == 1 and setters.has_key('doc'):
+        if len(setters) == 1 and 'doc' in setters:
             continue
-        if d.has_key(var):
+        if var in d:
             if isinstance(d[var], (types.MethodType, types.FunctionType)):
                 warnings.warn(
                     "I tried to set the property %r, but it was "
@@ -120,7 +119,7 @@ def unmakeProperties(obj):
     for var, value in d.items():
         if isinstance(value, property):
             for prop in [value.fget, value.fset, value.fdel]:
-                if prop and not d.has_key(prop.__name__):
+                if prop and not prop.__name__ in d:
                     delFunc(obj, var)
                     break
 
@@ -235,22 +234,25 @@ class sqlmeta(object):
     # Does the row require syncing?
     dirty = False
 
+    # Default encoding for UnicodeCol's
+    dbEncoding = None
+
     __metaclass__ = declarative.DeclarativeMeta
 
     def __classinit__(cls, new_attrs):
         for attr in cls._unshared_attributes:
-            if not new_attrs.has_key(attr):
+            if attr not in new_attrs:
                 setattr(cls, attr, None)
         declarative.setup_attributes(cls, new_attrs)
 
     def __init__(self, instance):
         self.instance = weakref.proxy(instance)
 
+    @classmethod
     def send(cls, signal, *args, **kw):
         events.send(signal, cls.soClass, *args, **kw)
 
-    send = classmethod(send)
-
+    @classmethod
     def setClass(cls, soClass):
         cls.soClass = soClass
         if not cls.style:
@@ -289,8 +291,6 @@ class sqlmeta(object):
         cls.joins = []
         cls.joinDefinitions = cls.joinDefinitions[:]
 
-    setClass = classmethod(setClass)
-
     ############################################################
     ## Adding special values, like columns and indexes
     ############################################################
@@ -299,6 +299,7 @@ class sqlmeta(object):
     ## Column handling
     ########################################
 
+    @classmethod
     def addColumn(cls, columnDef, changeSchema=False, connection=None):
         post_funcs = []
         cls.send(events.AddColumnSignal, cls.soClass, connection,
@@ -434,8 +435,7 @@ class sqlmeta(object):
         for func in post_funcs:
             func(soClass, column)
 
-    addColumn = classmethod(addColumn)
-
+    @classmethod
     def addColumnsFromDatabase(sqlmeta, connection=None):
         soClass = sqlmeta.soClass
         conn = connection or soClass._connection
@@ -445,8 +445,7 @@ class sqlmeta(object):
                     columnDef.name = columnDef.name.encode('ascii')
                 sqlmeta.addColumn(columnDef)
 
-    addColumnsFromDatabase = classmethod(addColumnsFromDatabase)
-
+    @classmethod
     def delColumn(cls, column, changeSchema=False, connection=None):
         sqlmeta = cls
         soClass = sqlmeta.soClass
@@ -468,17 +467,17 @@ class sqlmeta(object):
         del sqlmeta.columnDefinitions[name]
         sqlmeta.columnList.remove(column)
         delattr(soClass, rawGetterName(name))
-        if sqlmeta._plainGetters.has_key(name):
+        if name in sqlmeta._plainGetters:
             delattr(soClass, getterName(name))
         delattr(soClass, rawSetterName(name))
-        if sqlmeta._plainSetters.has_key(name):
+        if name in sqlmeta._plainSetters:
             delattr(soClass, setterName(name))
         if column.foreignKey:
             delattr(soClass, rawGetterName(soClass.sqlmeta.style.instanceIDAttrToAttr(name)))
-            if sqlmeta._plainForeignGetters.has_key(name):
+            if name in sqlmeta._plainForeignGetters:
                 delattr(soClass, getterName(name))
             delattr(soClass, rawSetterName(soClass.sqlmeta.style.instanceIDAttrToAttr(name)))
-            if sqlmeta._plainForeignSetters.has_key(name):
+            if name in sqlmeta._plainForeignSetters:
                 delattr(soClass, setterName(name))
         if column.alternateMethodName:
             delattr(soClass, column.alternateMethodName)
@@ -494,12 +493,11 @@ class sqlmeta(object):
         for func in post_funcs:
             func(soClass, column)
 
-    delColumn = classmethod(delColumn)
-
     ########################################
     ## Join handling
     ########################################
 
+    @classmethod
     def addJoin(cls, joinDef):
         sqlmeta = cls
         soClass = cls.soClass
@@ -548,8 +546,7 @@ class sqlmeta(object):
         if soClass._SO_finishedClassCreation:
             makeProperties(soClass)
 
-    addJoin = classmethod(addJoin)
-
+    @classmethod
     def delJoin(sqlmeta, joinDef):
         soClass = sqlmeta.soClass
         for join in sqlmeta.joins:
@@ -571,41 +568,39 @@ class sqlmeta(object):
                 # by index.
                 sqlmeta.joins[i] = None
         delattr(soClass, rawGetterName(meth))
-        if sqlmeta._plainJoinGetters.has_key(meth):
+        if meth in sqlmeta._plainJoinGetters:
             delattr(soClass, getterName(meth))
         if hasattr(join, 'remove'):
             delattr(soClass, '_SO_remove' + join.addRemovePrefix)
-            if sqlmeta._plainJoinRemovers.has_key(meth):
+            if meth in sqlmeta._plainJoinRemovers:
                 delattr(soClass, 'remove' + join.addRemovePrefix)
         if hasattr(join, 'add'):
             delattr(soClass, '_SO_add' + join.addRemovePrefix)
-            if sqlmeta._plainJoinAdders.has_key(meth):
+            if meth in sqlmeta._plainJoinAdders:
                 delattr(soClass, 'add' + join.addRemovePrefix)
 
         if soClass._SO_finishedClassCreation:
             unmakeProperties(soClass)
             makeProperties(soClass)
 
-    delJoin = classmethod(delJoin)
-
     ########################################
     ## Indexes
     ########################################
 
+    @classmethod
     def addIndex(cls, indexDef):
         cls.indexDefinitions.append(indexDef)
         index = indexDef.withClass(cls.soClass)
         cls.indexes.append(index)
         setattr(cls.soClass, index.name, index)
-    addIndex = classmethod(addIndex)
 
     ########################################
     ## Utility methods
     ########################################
 
+    @classmethod
     def getColumns(sqlmeta):
         return sqlmeta.columns.copy()
-    getColumns = classmethod(getColumns)
 
     def asDict(self):
         """
@@ -617,6 +612,7 @@ class sqlmeta(object):
         result['id'] = self.instance.id
         return result
 
+    @classmethod
     def expireAll(sqlmeta, connection=None):
         """
         Expire all instances of this class.
@@ -628,7 +624,6 @@ class sqlmeta(object):
         for item in cache_set.getAll(soClass):
             item.expire()
 
-    expireAll = classmethod(expireAll)
 
 sqlhub = dbconnection.ConnectionHub()
 
@@ -736,11 +731,11 @@ class SQLObject(object):
         if not is_base:
             cls._SO_cleanDeprecatedAttrs(new_attrs)
 
-        if new_attrs.has_key('_connection'):
+        if '_connection' in new_attrs:
             connection = new_attrs['_connection']
             del cls._connection
-            assert not new_attrs.has_key('connection')
-        elif new_attrs.has_key('connection'):
+            assert 'connection' not in new_attrs
+        elif 'connection' in new_attrs:
             connection = new_attrs['connection']
             del cls.connection
         else:
@@ -817,7 +812,7 @@ class SQLObject(object):
 
         classregistry.registry(sqlmeta.registry).addClass(cls)
 
-    # @classmethod
+    @classmethod
     def _SO_setupSqlmeta(cls, new_attrs, is_base):
         """
         This fixes up the sqlmeta attribute.  It handles both the case
@@ -826,7 +821,7 @@ class SQLObject(object):
         inheritance.  Lastly it calls sqlmeta.setClass, which handles
         much of the setup.
         """
-        if (not new_attrs.has_key('sqlmeta')
+        if ('sqlmeta' not in new_attrs
             and not is_base):
             # We have to create our own subclass, usually.
             # type(className, bases_tuple, attr_dict) creates a new subclass.
@@ -859,9 +854,7 @@ class SQLObject(object):
         if not is_base: # Do not pollute the base sqlmeta class
             cls.sqlmeta.setClass(cls)
 
-    _SO_setupSqlmeta = classmethod(_SO_setupSqlmeta)
-
-    # @classmethod
+    @classmethod
     def _SO_cleanDeprecatedAttrs(cls, new_attrs):
         """
         This removes attributes on SQLObject subclasses that have
@@ -869,13 +862,12 @@ class SQLObject(object):
         a deprecation warning is given.
         """
         for attr in ():
-            if new_attrs.has_key(attr):
+            if attr in new_attrs:
                 deprecated("%r is deprecated and read-only; please do "
                            "not use it in your classes until it is fully "
                            "deprecated" % attr, level=1, stacklevel=5)
 
-    _SO_cleanDeprecatedAttrs = classmethod(_SO_cleanDeprecatedAttrs)
-
+    @classmethod
     def get(cls, id, connection=None, selectResults=None):
 
         assert id is not None, 'None is not a possible id for %s' % cls.__name__
@@ -907,11 +899,9 @@ class SQLObject(object):
                 val._SO_writeLock.release()
         return val
 
-    get = classmethod(get)
-
+    @classmethod
     def _notifyFinishClassCreation(cls):
         pass
-    _notifyFinishClassCreation = classmethod(_notifyFinishClassCreation)
 
     def _init(self, id, connection=None, selectResults=None):
         assert id is not None
@@ -1069,7 +1059,7 @@ class SQLObject(object):
         # Filter out items that don't map to column names.
         # Those will be set directly on the object using
         # setattr(obj, name, value).
-        is_column = self.sqlmeta._plainSetters.has_key
+        is_column = lambda _c: _c in self.sqlmeta._plainSetters
         f_is_column = lambda item: is_column(item[0])
         f_not_column = lambda item: not is_column(item[0])
         items = kw.items()
@@ -1203,14 +1193,14 @@ class SQLObject(object):
             # The get() classmethod/constructor uses a magic keyword
             # argument when it wants an empty object, fetched from the
             # database.  So we have nothing more to do in that case:
-            if kw.has_key('_SO_fetch_no_create'):
+            if '_SO_fetch_no_create' in kw:
                 return
 
             post_funcs = []
             self.sqlmeta.send(events.RowCreateSignal, self, kw, post_funcs)
 
             # Pass the connection object along if we were given one.
-            if kw.has_key('connection'):
+            if 'connection' in kw:
                 connection = kw.pop('connection')
                 if getattr(self, '_connection', None) is not connection:
                     self._connection = connection
@@ -1218,7 +1208,7 @@ class SQLObject(object):
 
             self._SO_writeLock = threading.Lock()
 
-            if kw.has_key('id'):
+            if 'id' in kw:
                 id = self.sqlmeta.idType(kw['id'])
                 del kw['id']
             else:
@@ -1251,7 +1241,7 @@ class SQLObject(object):
 
             # Then we check if the column wasn't passed in, and
             # if not we try to get the default.
-            if not kw.has_key(column.name) and not kw.has_key(column.foreignName):
+            if column.name not in kw and column.foreignName not in kw:
                 default = column.default
 
                 # If we don't get it, it's an error:
@@ -1311,6 +1301,7 @@ class SQLObject(object):
     def _SO_getID(self, obj):
         return getID(obj)
 
+    @classmethod
     def _findAlternateID(cls, name, dbName, value, connection=None):
         if isinstance(name, str):
             name = (name,)
@@ -1329,8 +1320,8 @@ class SQLObject(object):
             [cls.sqlmeta.idName] +
             [column.dbName for column in cls.sqlmeta.columnList],
             condition), None
-    _findAlternateID = classmethod(_findAlternateID)
 
+    @classmethod
     def _SO_fetchAlternateID(cls, name, dbName, value, connection=None, idxName=None):
         result, obj = cls._findAlternateID(name, dbName, value, connection)
         if not result:
@@ -1343,18 +1334,18 @@ class SQLObject(object):
                 names = ', '.join(names)
                 raise SQLObjectNotFound, "The %s by unique index %s(%s) does not exist" % (cls.__name__, idxName, names)
         if obj:
-           return obj
+            return obj
         if connection:
             obj = cls.get(result[0], connection=connection, selectResults=result[1:])
         else:
             obj = cls.get(result[0], selectResults=result[1:])
         return obj
-    _SO_fetchAlternateID = classmethod(_SO_fetchAlternateID)
 
+    @classmethod
     def _SO_depends(cls):
         return findDependencies(cls.__name__, cls.sqlmeta.registry)
-    _SO_depends = classmethod(_SO_depends)
 
+    @classmethod
     def select(cls, clause=None, clauseTables=None,
                orderBy=NoDefault, limit=None,
                lazyColumns=False, reversed=False,
@@ -1369,22 +1360,20 @@ class SQLObject(object):
                              distinct=distinct,
                              connection=connection,
                              join=join, forUpdate=forUpdate)
-    select = classmethod(select)
 
+    @classmethod
     def selectBy(cls, connection=None, **kw):
         conn = connection or cls._connection
         return cls.SelectResultsClass(cls,
                              conn._SO_columnClause(cls, kw),
                              connection=conn)
 
-    selectBy = classmethod(selectBy)
-
+    @classmethod
     def tableExists(cls, connection=None):
         conn = connection or cls._connection
         return conn.tableExists(cls.sqlmeta.table)
 
-    tableExists = classmethod(tableExists)
-
+    @classmethod
     def dropTable(cls, ifExists=False, dropJoinTables=True, cascade=False,
                   connection=None):
         conn = connection or cls._connection
@@ -1401,8 +1390,8 @@ class SQLObject(object):
             connection.query(sql)
         for func in post_funcs:
             func(cls, conn)
-    dropTable = classmethod(dropTable)
 
+    @classmethod
     def createTable(cls, ifNotExists=False, createJoinTables=True,
                     createIndexes=True, applyConstraints=True,
                     connection=None):
@@ -1428,8 +1417,8 @@ class SQLObject(object):
         for func in post_funcs:
             func(cls, conn)
         return extra_sql
-    createTable = classmethod(createTable)
 
+    @classmethod
     def createTableSQL(cls, createJoinTables=True, createIndexes=True,
                        connection=None):
         conn = connection or cls._connection
@@ -1443,8 +1432,8 @@ class SQLObject(object):
             if index_sql:
                 sql += ';\n' + index_sql
         return sql, constraints
-    createTableSQL = classmethod(createTableSQL)
 
+    @classmethod
     def createJoinTables(cls, ifNotExists=False, connection=None):
         conn = connection or cls._connection
         for join in cls._getJoinsToCreate():
@@ -1452,24 +1441,24 @@ class SQLObject(object):
                 conn.tableExists(join.intermediateTable)):
                 continue
             conn._SO_createJoinTable(join)
-    createJoinTables = classmethod(createJoinTables)
 
+    @classmethod
     def createJoinTablesSQL(cls, connection=None):
         conn = connection or cls._connection
         sql = []
         for join in cls._getJoinsToCreate():
             sql.append(conn._SO_createJoinTableSQL(join))
         return ';\n'.join(sql)
-    createJoinTablesSQL = classmethod(createJoinTablesSQL)
 
+    @classmethod
     def createIndexes(cls, ifNotExists=False, connection=None):
         conn = connection or cls._connection
         for index in cls.sqlmeta.indexes:
             if not index:
                 continue
             conn._SO_createIndex(cls, index)
-    createIndexes = classmethod(createIndexes)
 
+    @classmethod
     def createIndexesSQL(cls, connection=None):
         conn = connection or cls._connection
         sql = []
@@ -1478,8 +1467,8 @@ class SQLObject(object):
                 continue
             sql.append(conn.createIndexSQL(cls, index))
         return ';\n'.join(sql)
-    createIndexesSQL = classmethod(createIndexesSQL)
 
+    @classmethod
     def _getJoinsToCreate(cls):
         joins = []
         for join in cls.sqlmeta.joins:
@@ -1491,8 +1480,8 @@ class SQLObject(object):
                 continue
             joins.append(join)
         return joins
-    _getJoinsToCreate = classmethod(_getJoinsToCreate)
 
+    @classmethod
     def dropJoinTables(cls, ifExists=False, connection=None):
         conn = connection or cls._connection
         for join in cls.sqlmeta.joins:
@@ -1507,8 +1496,7 @@ class SQLObject(object):
                 continue
             conn._SO_dropJoinTable(join)
 
-    dropJoinTables = classmethod(dropJoinTables)
-
+    @classmethod
     def clearTable(cls, connection=None, clearJoinTables=True):
         # 3-03 @@: Maybe this should check the cache... but it's
         # kind of crude anyway, so...
@@ -1517,7 +1505,6 @@ class SQLObject(object):
         if clearJoinTables:
             for join in cls._getJoinsToCreate():
                 conn.clearTable(join.intermediateTable)
-    clearTable = classmethod(clearTable)
 
     def destroySelf(self):
         post_funcs = []
@@ -1538,8 +1525,8 @@ class SQLObject(object):
             # Free related joins
             for join in k.sqlmeta.joins:
                 if isinstance(join, joins.SORelatedJoin) and join.otherClassName == klass.__name__:
-                     q = "DELETE FROM %s WHERE %s=%d" % (join.intermediateTable, join.otherColumn, self.id)
-                     self._connection.query(q)
+                    q = "DELETE FROM %s WHERE %s=%d" % (join.intermediateTable, join.otherColumn, self.id)
+                    self._connection.query(q)
 
             cols = findDependantColumns(klass.__name__, k)
 
@@ -1591,25 +1578,22 @@ class SQLObject(object):
         for func in post_funcs:
             func(self)
 
+    @classmethod
     def delete(cls, id, connection=None):
         obj = cls.get(id, connection=connection)
         obj.destroySelf()
 
-    delete = classmethod(delete)
-
+    @classmethod
     def deleteMany(cls, where=NoDefault, connection=None):
         conn = connection or cls._connection
         conn.query(conn.sqlrepr(sqlbuilder.Delete(cls.sqlmeta.table, where)))
 
-    deleteMany = classmethod(deleteMany)
-
+    @classmethod
     def deleteBy(cls, connection=None, **kw):
         conn = connection or cls._connection
         conn.query(conn.sqlrepr(sqlbuilder.Delete(cls.sqlmeta.table,
             conn._SO_columnClause(cls, kw))))
 
-    deleteBy = classmethod(deleteBy)
-
     def __repr__(self):
         if not hasattr(self, 'id'):
             # Object initialization not finished.  No attributes can be read.
@@ -1622,19 +1606,17 @@ class SQLObject(object):
     def __sqlrepr__(self, db):
         return str(self.id)
 
+    @classmethod
     def sqlrepr(cls, value, connection=None):
         return (connection or cls._connection).sqlrepr(value)
 
-    sqlrepr = classmethod(sqlrepr)
-
+    @classmethod
     def coerceID(cls, value):
         if isinstance(value, cls):
             return value.id
         else:
             return cls.sqlmeta.idType(value)
 
-    coerceID = classmethod(coerceID)
-
     def _reprItems(self):
         items = []
         for col in self.sqlmeta.columnList:
@@ -1645,13 +1627,12 @@ class SQLObject(object):
             items.append((col.name, value))
         return items
 
+    @classmethod
     def setConnection(cls, value):
         if isinstance(value, basestring):
             value = dbconnection.connectionForURI(value)
         cls._connection = value
 
-    setConnection = classmethod(setConnection)
-
     def tablesUsedImmediate(self):
         return [self.__class__.q]
 
@@ -1668,16 +1649,16 @@ class SQLObject(object):
         return not self.__eq__(other)
 
     def __lt__(self, other):
-       return NotImplemented
+        return NotImplemented
 
     def __le__(self, other):
-       return NotImplemented
+        return NotImplemented
 
     def __gt__(self, other):
-       return NotImplemented
+        return NotImplemented
 
     def __ge__(self, other):
-       return NotImplemented
+        return NotImplemented
 
 
     def __getstate__(self):
