@@ -1,4 +1,6 @@
-from sqlobject.sqlbuilder import sqlrepr, TRUE, FALSE
+import sys
+from datetime import timedelta
+from sqlobject.sqlbuilder import sqlrepr
 from sqlobject.sqlbuilder import SQLExpression, SQLObjectField, \
      Select, Insert, Update, Delete, Replace, \
      SQLTrueClauseClass, SQLConstant, SQLPrefix, SQLCall, SQLOp
@@ -55,7 +57,7 @@ def test_string_000():
     assert sqlrepr('A String\000Another', 'postgres') == "'A String\\0Another'"
 
 def test_string_():
-    assert sqlrepr('A String\'Another', 'postgres') == "'A String\\'Another'"
+    assert sqlrepr('A String\tAnother', 'postgres') == "'A String\\tAnother'"
     assert sqlrepr('A String\'Another', 'firebird') == "'A String''Another'"
 
 def test_simple_unicode():
@@ -77,10 +79,10 @@ def test_tuple():
     assert sqlrepr(('one','two','three'), 'postgres') == "('one', 'two', 'three')"
 
 def test_bool():
-    assert sqlrepr(TRUE, 'postgres') == "'t'"
-    assert sqlrepr(FALSE, 'postgres') == "'f'"
-    assert sqlrepr(TRUE, 'mysql') == "1"
-    assert sqlrepr(FALSE, 'mysql') == "0"
+    assert sqlrepr(True, 'postgres') == "'t'"
+    assert sqlrepr(False, 'postgres') == "'f'"
+    assert sqlrepr(True, 'mysql') == "1"
+    assert sqlrepr(False, 'mysql') == "0"
 
 def test_datetime():
     from datetime import datetime, date, time
@@ -104,7 +106,7 @@ def test_sqlexpr():
     assert sqlrepr(instance) == repr(instance)
 
 def test_sqlobjectfield():
-    instance = SQLObjectField('test', 'test', 'test')
+    instance = SQLObjectField('test', 'test', 'test', None, None)
     assert sqlrepr(instance) == repr(instance)
 
 def test_select():
@@ -187,9 +189,14 @@ def test_sets():
         pass
     else:
         assert sqlrepr(set([1])) == "(1)"
-    try:
-        from sets import Set
-    except ImportError:
-        pass
-    else:
-        assert sqlrepr(Set([1])) == "(1)"
+    if sys.version_info[:3] < (2, 6, 0): # Module sets was deprecated in Python 2.6
+        try:
+            from sets import Set
+        except ImportError:
+            pass
+        else:
+            assert sqlrepr(Set([1])) == "(1)"
+
+def test_timedelta():
+    assert sqlrepr(timedelta(seconds=30*60)) == \
+        "INTERVAL '0 days 1800 seconds'"
